'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stream = require('stream');

var _vinyl = require('vinyl');

var _vinyl2 = _interopRequireDefault(_vinyl);

var _helpers = require('./helpers');

var _index = require('../index');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _cache = require('../../cache');

var _cache2 = _interopRequireDefault(_cache);

var _bridge = require('../../bridge');

var _bridge2 = _interopRequireDefault(_bridge);

var _fns = require('../../lib/fns');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// time we wait for browser load before we just force push
const UPPER_WAIT_LIMIT = 1000;
const isFileType = (_path, ext) => _fns.path.extname(_path) == `.${ ext }`;
const debug = _fns.log.bind(null, { name: 'stream', icon: '🎏' });

// socket for sending files to browser from editor
// has a lock system that waits before sending more

class SuperStream {
  constructor() {
    this.basePath = (0, _opts2.default)('appDir');
    this.relPath = p => _fns.path.relative(this.basePath, p);
    this.isBuilding = {};
    this.upperLimit = {};
    this.queue = {};
    this.stream = new _stream.Readable({ objectMode: true });
    this.stream._read = function (n) {};

    _bridge2.default.onDidReceiveMessage('live:save', _fns._.throttle(this.fileSend.bind(this), 16, { leading: true }));

    _fns.emitter.on('script:end', _ref => {
      let path = _ref.path;
      return this.doneBuilding(path);
    });
    (0, _index.event)('error', _ref2 => {
      let path = _ref2.path;
      return this.doneBuilding(path);
    });
  }

  getStream() {
    return this.stream;
  }

  doneBuilding(path) {
    const rel = this.relPath(path);
    this.isBuilding[rel] = false;
    debug('IN', 'done building', rel);
    this.runQueue(rel);
  }

  // prevent upper limit send post hoc
  avoidSending(path) {
    const rel = this.relPath(path);
    this.queue[rel] = false;
    clearTimeout(this.upperLimit[rel]);
  }

  runQueue(rel) {
    const queued = this.queue[rel];
    this.queue[rel] = false;
    if (queued) queued();
  }

  fileSend(_ref3) {
    let path = _ref3.path;
    let startTime = _ref3.startTime;
    let contents = _ref3.contents;

    const rel = this.relPath(path);
    debug('SIN', rel);

    // check if file actually in motion project
    if (!path || path.indexOf(this.basePath) !== 0 || rel.indexOf('.motion') >= 0 || !isFileType(path, 'js')) {
      debug('  file not streamable', path.indexOf(this.basePath) !== 0, rel.indexOf('.motion') === 0, !isFileType(path, 'js'));
      return;
    }

    const sendImmediate = _cache2.default.isInternal(rel) || _cache2.default.isInstalling(rel);

    this.pushStreamRun(rel, () => {
      debug('SOUT', rel);
      this.isBuilding[rel] = true;
      const file = new _vinyl2.default((0, _fns.vinyl)(this.basePath, path, new Buffer(contents)));

      const stackTime = [{
        name: 'fileSend',
        time: +Date.now() - startTime
      }];

      Object.assign(file, { startTime, stackTime });

      this.stream.push(file);
    }, sendImmediate);
  }

  pushStreamRun(rel, finish, sendImmediate) {
    if (!this.isBuilding[rel] || sendImmediate) return finish();

    this.queue[rel] = finish;

    // ensure upper limit on wait
    clearTimeout(this.upperLimit[rel]);
    this.upperLimit[rel] = setTimeout(() => {
      if (!this.queue[rel]) return;
      debug('upper limit! finish'.yellow);
      finish();
    }, UPPER_WAIT_LIMIT);
  }
}
exports.default = SuperStream;
//# sourceMappingURL=superStream.js.map